@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 ##############################################################
 # smscon_service - remote control daemon for Nokia N900 phone#
-# $Id: smscon_service.py 156 2012-01-20 20:12:07Z yablacky $
+# $Id: smscon_service.py 164 2012-01-22 22:32:04Z yablacky $
 ##############################################################
 
 # standard modules
@@ -527,7 +527,7 @@ def ProcessCommand(Command, SenderPhoneNumber=None, CommandOrigin=None, ExtraInf
     CommandList = [smscnf[cmd] for cmd in smscnf.getSmsCommandList() if smscnf[cmd] != ""]
     if not Command in CommandList:
         if LogIfCmdUnknown:
-            LOGGER.error( '"%s" was not a valid %s command.' % (Command, smscon.NAME) )
+            LOGGER.error( '"%s" is not a valid %s command.' % (Command, smscon.NAME) )
             LogBatteryChargeInfo()
         return False
 
@@ -1440,10 +1440,10 @@ def DisableSSHcontrol(Verbose = True):
 
 def EnableGpsAcquireMode(Mode):
     """
-    Start the GPS device mainloop.
+    Schedule starting the GPS device.
     """
     LOGGER.debug('Enabling GPS device for %s mode (current mode is %s).' % (Mode, GpsMode))
-    gobject.idle_add(GPSstart, Mode)
+    gobject.idle_add(OnGPSstart, Mode)
 
 def DisableGpsAcquiring():
     """
@@ -1452,16 +1452,17 @@ def DisableGpsAcquiring():
     global GpsMode, GpsControl
 
     if GpsMode:
-        LOGGER.info('Stopped GPS device from %s mode.' % GpsMode)
+        LOGGER.info('Stopping GPS device from %s mode.' % GpsMode)
         GpsMode = False
         # stop the GPS device
         GpsControl.stop()
     else:
         LOGGER.info('GPS device already stopped.')
 
-def GPSstart(Mode):
+def OnGPSstart(Mode):
     """
-    Start the GPS device in given Mode or switch already started device to given Mode.
+    Callback to start the GPS device in given Mode or switch already started device to given Mode.
+    @return False
     """
     global GpsMode, GpsControl, GpsStartTime, GpsTimeoutTime, GpsLocationBasisTime, GspTrackingBasisTime
 
@@ -1476,9 +1477,10 @@ def GPSstart(Mode):
         GpsControl.start()
     return False    # single shot.
 
-def OnGPSstop(GpsControl, GpsData):
+def OnGPSstop(GpsControl, notUsed):
     """
-    Stop the GPS device.
+    Callback from GPS subsystem in case GPS device got stopped.
+    @return False
     """
     global GpsMode
     GpsMode = False
@@ -1487,13 +1489,12 @@ def OnGPSstop(GpsControl, GpsData):
     if GpsIsTest:
         GpsIsTest = False
         RestoreNetworkSettings()
-    
-    LOGGER.info('Stopping GPS device.')
-    GpsData.quit()
+    return False    # single shot.
 
-def OnGPSchanged(GpsDevice, GpsData):
+def OnGPSchanged(GpsDevice, GpsControl):
     """
-    Get GPS device data.
+    Callback from GPS subsystem in case GPS device data changed.
+    @return void
     """
     global GpsCoordinateList, GpsStartTime, GpsTimeoutTime, GpsLocationBasisTime, GspTrackingBasisTime
 
@@ -1503,12 +1504,11 @@ def OnGPSchanged(GpsDevice, GpsData):
         return
 
     timeNow = time.time()
-
     StartedSinceSeconds = timeNow - GpsStartTime
     TimeoutSeconds      = timeNow - GpsTimeoutTime
     LOGGER.debug('GpsStartTime = %s, StartedSinceSeconds = %.1f, TimeoutSeconds = %.1f'
                                 % (GpsStartTime, StartedSinceSeconds, TimeoutSeconds))
-    LOGGER.debug('GpsDevice.status = %s' % GpsDevice.status) # LOCATION_GPS_DEVICE_STATUS_NO_FIX  or  LOCATION_GPS_DEVICE_STATUS_FIX
+    LOGGER.debug('GpsDevice.status = %s' % GpsDevice.status) # location.GPS_DEVICE_STATUS_NO_FIX  or  location.GPS_DEVICE_STATUS_FIX
 
     # Check if data is OK and log why it it not OK.
 
@@ -1534,6 +1534,14 @@ def OnGPSchanged(GpsDevice, GpsData):
         if not GpsDataOK:
             LOGGER.info('Waiting for valid GPS coordinate (since %.1f of %d sec).' % (TimeoutSeconds, int(smscnf.GPSTIMEOUT)))
 
+    if GpsDataOK and GpsDevice.status == location.GPS_DEVICE_STATUS_NO_FIX:
+        # Ignore those coordinates; they are valid but might be just the last coordinate
+        # from last long-time-ago fix and has nothing to do with current position:
+        GpsDataOK = False
+        LOGGER.info('Artificial GPS coordinate (%f, %f, %.1f m.) after %.1f sec since start. Ignored.'
+                                % (Latitude, Longitude, Accuracy, StartedSinceSeconds) )
+    # Check timeout condition
+
     if GpsDataOK:
         GpsTimeoutTime = timeNow  # restart timeout
     IsTimeoutDetected = TimeoutSeconds > int(smscnf.GPSTIMEOUT)
@@ -1545,15 +1553,10 @@ def OnGPSchanged(GpsDevice, GpsData):
         LocationSinceSeconds = timeNow - GpsLocationBasisTime 
         if GpsDataOK:
 
-            if type(GpsCoordinateList).__name__ != 'list':
+            if type(GpsCoordinateList).__name__ != 'list' or len(GpsCoordinateList) == 0:
                 GpsCoordinateList = []
                 GpsLocationBasisTime = timeNow
                 LOGGER.info('Start collecting min. %s GPS coordinate(s) for max. %d sec...' % (smscnf.GPSPOLLING, LocationTimeoutSeconds))
-                # Ignore the 1st received coordinate because this might be just the last coordinate
-                # from last long-time-ago fix and has nothing to do with current position.  
-                LOGGER.info('GPS coordinate #%d (%f, %f, %.1f m.) after %.1f sec since start. Ignored.'
-                                % (len(GpsCoordinateList), Latitude, Longitude, Accuracy, StartedSinceSeconds) )
-                return  # gather more coordinates
 
             GpsCoordinateList.append( (Latitude, Longitude, Accuracy) )
             LOGGER.info('GPS coordinate #%d (%f, %f, %.1f m.) after %.1f sec since fix.'
@@ -1586,7 +1589,7 @@ def OnGPSchanged(GpsDevice, GpsData):
                             % (len(GpsCoordinateList), Latitude, Longitude, Accuracy, StartedSinceSeconds) )
             GpsCoordinateList = None
             # stop the GPS device
-            GpsData.stop()
+            GpsControl.stop()
 
     elif GpsMode == GpsModeTRACKING:
         if GpsDataOK:
@@ -1604,7 +1607,7 @@ def OnGPSchanged(GpsDevice, GpsData):
             LOGGER.error(     'GPS device timeout after %.1f sec.' % StartedSinceSeconds)
             RevealThat('GPS', 'GPS device timeout after %.1f sec.' % StartedSinceSeconds, smscnf.GPSSEND)                    
             # stop the GPS device
-            GpsData.stop()
+            GpsControl.stop()
         LOGGER.debug('gather more coordinates (data not OK but no timeout).')
         return
 
@@ -1620,28 +1623,29 @@ def OnGPSchanged(GpsDevice, GpsData):
                 Attachment = smscnf.GPSSEND in ['email', 'both'] and CreateGoogleMap(Latitude, Longitude),
                 Attachhint = 'image')
 
-def OnGPSerror(GpsControl, GpsError, GpsData):
+def OnGPSerror(GpsControl, GpsError, notUsed):
     """
-    GPS device error detected.
+    Callback from GPS subystem to handle GPS device error detected.
     """
-    global GpsMode
-    GpsMode = False
 
     if GpsError == location.ERROR_USER_REJECTED_DIALOG:
-        LOGGER.error("GPS device error (User didn't enable requested methods).")
+        ErrorMsg = ("GPS device error (User didn't enable requested methods).")
     elif GpsError == location.ERROR_USER_REJECTED_SETTINGS:
-        LOGGER.error('GPS device error (User changed settings, which disabled location).')
+        ErrorMsg =('GPS device error (User changed settings, which disabled location).')
     elif GpsError == location.ERROR_BT_GPS_NOT_AVAILABLE:
-        LOGGER.error('GPS device error (Problems with BT GPS).')
+        ErrorMsg =('GPS device error (Problems with BT GPS).')
     elif GpsError == location.ERROR_METHOD_NOT_ALLOWED_IN_OFFLINE_MODE:
-        LOGGER.error('GPS device error (Requested method is not allowed in offline mode).')
+        ErrorMsg =('GPS device error (Requested method is not allowed in offline mode).')
     elif GpsError == location.ERROR_SYSTEM:
-        LOGGER.error('GPS device error (System error).')
+        ErrorMsg =('GPS device error (System error).')
     else:
-        LOGGER.error('GPS device error (Unknown error).')
-        
+        ErrorMsg =('GPS device error (Unknown error %s).' % str(GpsError))
+    LOGGER.error(ErrorMsg)
+
     LOGGER.error('Stopping failed GPS device.')
-    GpsData.quit()
+    DisableGpsAcquiring()
+
+    RevealThat('GPS Error', ErrorMsg)
 
 ###############################
 
@@ -2182,7 +2186,7 @@ def LoadConfiguration():
 def OnLocalCommandAvailable():
     """
     Try to read commands from local command pipe and execute them.
-    @return void  
+    @return False
     """
     global daemon
 
@@ -2197,6 +2201,7 @@ def OnLocalCommandAvailable():
             LOGGER.info("----- TEST '%s' DONE -----" % ' '.join(cliArgs))
         else:
             LOGGER.warning('Unknown command "%s" found in local command channel. Ignored.' % Command)
+    return False
 
 def main():
     """
@@ -2213,17 +2218,8 @@ def main():
                     lambda signum, stack_frame: ExitDaemon("on signal %s" % str(signum))  )
     signal.signal(smscon.smscon_daemon.TELL_RELOAD,
                     lambda signum, stack_frame: LoadConfiguration()                       )
-
-    def commandSignalHandler(signum, stack_frame):
-        """
-        Signal handler to prevent OnLocalCommandAvailable() itself being interrupted
-        by command signals. So it can work off all pushed in commands sequentially.  
-        """
-        oldhandler = signal.signal(signum, signal.SIG_IGN)
-        OnLocalCommandAvailable()
-        signal.signal(signum, oldhandler)
-
-    signal.signal(smscon.smscon_daemon.TELL_COMMAND, commandSignalHandler)
+    signal.signal(smscon.smscon_daemon.TELL_COMMAND,
+                    lambda signum, stack_frame: gobject.idle_add(OnLocalCommandAvailable) )
     
     LoadConfiguration() or ExitDaemon('on config load error')
     ValidateIMSI()
@@ -2252,9 +2248,9 @@ def main():
         GpsDevice  = location.GPSDevice()
 
         # set GPS signal receivers (need them for DoCommandLine())
-        GpsControl.connect('error-verbose', OnGPSerror, gobject.MainLoop() )
+        GpsControl.connect('error-verbose', OnGPSerror, None )
         GpsDevice.connect('changed', OnGPSchanged, GpsControl )
-        GpsControl.connect('gpsd-stopped', OnGPSstop, gobject.MainLoop() )
+        GpsControl.connect('gpsd-stopped', OnGPSstop, None )
 
         ApplyConfiguration()
 
